import re

import pandas as pd

from crossborder.utils.base_country_code import COUNTRY_CODE_MAPPING
from crossborder.utils.constants import EXCLUDE_REGIONS
from crossborder.utils.parse_utils import clean_county_name

# 配置参数
EXCEL_PATH = r"D:/Downloads/2025051809262394128.xls"
OUTPUT_SQL = "../downloads/country_trade04.sql"


# 在代码中添加例外处理
def get_country_code(chinese_name):
    """带异常处理的国家编码查询"""
    code = COUNTRY_CODE_MAPPING.get(chinese_name.strip(), None)

    # 特殊处理逻辑
    if not code:
        if chinese_name.endswith("(地区)"):
            return "N/A"  # 标记为地区
        elif "国家联盟" in chinese_name:
            return "ORG"  # 标记为国际组织
    return code


def parse_excel_to_sql():
    """主处理函数"""
    # 读取Excel文件（注意需要xlrd 1.2.0版本）
    try:
        df = pd.read_excel(EXCEL_PATH, engine='xlrd', header=None, dtype=str)
    except ImportError:
        raise ImportError("需要xlrd库支持.xls文件，请安装：pip install xlrd==1.2.0")
    except Exception as e:
        raise ValueError(f"读取Excel文件失败：{str(e)}")

    # 解析年份月份
    year_month = None
    data_start = None
    for idx, row in df.iterrows():
        cell_value = str(row[1])
        if "进出口商品国别" in cell_value:
            match = re.search(r"(\d{4})年(\d{1,2})月", cell_value)
            if match:
                year = match.group(1)
                month = match.group(2).zfill(2)
                year_month = f"{year}-{month}"
                # 调整起始行计算公式
                data_start = idx + 6  # 原为+3，改为+4
                break

    if not year_month or data_start is None:
        raise ValueError("无法解析年份月份信息，请检查Excel文件格式")

    # 读取数据区域
    try:
        df_data = pd.read_excel(EXCEL_PATH, engine='xlrd', skiprows=data_start, header=None)
    except Exception as e:
        raise ValueError(f"读取数据区域失败：{str(e)}")

    # 列映射配置（根据实际Excel结构调整）
    COL_MAPPING = {
        "country_name": 1,
        "monthly_total": 2,
        "monthly_export": 4,
        "monthly_import": 6,
        "ytd_total": 3,
        "ytd_export": 5,
        "ytd_import": 7,
        "yoy_total": 8,
        "yoy_export": 9,
        "yoy_import": 10
    }

    # 生成SQL语句
    sql_statements = []
    for index, row in df_data.iterrows():
        try:
            # 获取国家名称
            country_name = str(row[COL_MAPPING["country_name"]]).strip()

            country_name = clean_county_name(country_name)

            # 跳过空行和区域数据
            if not country_name or country_name in EXCLUDE_REGIONS:
                continue

            # 获取国家代码
            country_code = get_country_code(country_name)
            # 新增：排除地区和组织代码
            if country_code in ['N/A', 'ORG', 'XX'] or not country_code:
                print(f"跳过地区/组织：{country_name}")
                continue
            if not country_code:
                print(f"警告：第{index + data_start + 1}行 未找到国家代码 [{country_name}]")
                continue

            # 数值处理函数（万元转元）
            # 修改后的数值处理函数
            def convert_value(val):
                """增强版数值转换，处理所有异常情况"""
                try:
                    # 处理特殊字符
                    cleaned = re.sub(r'[^0-9\.\-\+]', '', str(val))
                    # 处理空值和纯符号
                    if cleaned in ('', '-', '+'):
                        return 0.0
                    # 处理科学计数法
                    if 'e' in cleaned.lower():
                        return float(cleaned)

                    # 验证数值有效性
                    parts = cleaned.split('.')
                    if len(parts) > 2:  # 多个小数点
                        return 0.0
                    if '-' in cleaned and not cleaned.startswith('-'):  # 负号位置错误
                        return 0.0

                    return float(cleaned)
                except:
                    return 0.0

            def safe_convert(val):
                """安全转换函数，处理特殊字符和空值"""
                # 处理空值和特殊占位符
                if pd.isna(val) or str(val).strip() in ('', '-', 'NA'):
                    return 'null'  # 返回Python的None，对应SQL的NULL

                # 清理非数字字符
                cleaned = re.sub(r'[^0-9\.\-]', '', str(val))

                try:
                    return round(float(cleaned), 2)
                except:
                    return 'null'

            # 数据转换
            values = {
                "year_month": year_month,
                "country_code": country_code,
                "country_name": country_name.replace("'", "''"),  # 处理单引号
                "monthly_total": convert_value(row[COL_MAPPING["monthly_total"]]),
                "monthly_export": convert_value(row[COL_MAPPING["monthly_export"]]),
                "monthly_import": convert_value(row[COL_MAPPING["monthly_import"]]),
                "ytd_total": convert_value(row[COL_MAPPING["ytd_total"]]),
                "ytd_export": convert_value(row[COL_MAPPING["ytd_export"]]),
                "ytd_import": convert_value(row[COL_MAPPING["ytd_import"]]),
                "yoy_total": safe_convert(row[COL_MAPPING["yoy_total"]]),
                "yoy_export": safe_convert(row[COL_MAPPING["yoy_export"]]),
                "yoy_import": safe_convert(row[COL_MAPPING["yoy_import"]])
            }

            # 构建SQL语句
            sql = f"""INSERT INTO t_yujin_crossborder_country_trade (
                `year_month`, country_code, country_name,
                monthly_total, monthly_import, monthly_export,
                ytd_total, ytd_import, ytd_export,
                yoy_import_export, yoy_import, yoy_export
            ) VALUES (
                '{values["year_month"]}',
                '{values["country_code"]}',
                '{values["country_name"]}',
                {values["monthly_total"]},
                {values["monthly_import"]},
                {values["monthly_export"]},
                {values["ytd_total"]},
                {values["ytd_import"]},
                {values["ytd_export"]},
                {values["yoy_total"]},
                {values["yoy_export"]},
                {values["yoy_import"]}
            );"""
            sql_statements.append(sql)
        except Exception as e:
            print(f"第{index + data_start + 1}行处理失败：{str(e)}")
            continue

    # 写入SQL文件
    try:
        with open(OUTPUT_SQL, "w", encoding="utf-8") as f:
            f.write("\n".join(sql_statements))
        print(f"成功生成 {len(sql_statements)} 条SQL语句，保存至：{OUTPUT_SQL}")
    except Exception as e:
        print(f"文件写入失败：{str(e)}")


if __name__ == "__main__":
    try:
        parse_excel_to_sql()
    except Exception as e:
        print(f"程序执行失败：{str(e)}")