import os
import re
from decimal import Decimal
from pathlib import Path

import xlrd
from crossborder.utils.db_helper import DBHelper
from crossborder.utils.parse_utils import parse_value, extract_year_month_from_path
from crossborder.utils.constants import COUNTRY_CODE_MAPPING

# 全局状态变量（在函数内部使用）
current_class = None


def parse_hscode(row_value):
    """解析类/章信息（增强格式兼容性）"""
    global current_class

    cell_value = str(row_value).strip()
    clean_value = re.sub(r'\s+', ' ', cell_value)  # 合并连续空格

    # 匹配类格式：第X类（兼容空格）
    class_match = re.match(r'^第\s*(\d+)\s*类\b.*', clean_value)
    if class_match:
        class_num = class_match.group(1)
        current_class = f"{int(class_num):02d}"  # 两位数格式化
        return current_class

    # 匹配章格式：XX章（兼容前导零）
    chapter_match = re.match(r'^(\d+)\s*章\b.*', clean_value)
    if chapter_match and current_class:
        chapter_num = chapter_match.group(1).zfill(2)  # 自动补零
        return f"{current_class}{chapter_num}"

    return None


def parse_commodity_country_detail(file_path, trade_type):
    """处理商品国家贸易明细数据"""
    global current_class
    db = DBHelper()
    current_class = None

    # 从文件路径提取年份和月份
    year, month = extract_year_month_from_path(Path(file_path))
    year_month = f"{year}-{month:02d}"

    # 验证文件是否存在
    if not os.path.exists(file_path):
        print(f"⛔ 文件不存在：{file_path}")
        return 0

    try:
        workbook = xlrd.open_workbook(file_path)
        sheet = workbook.sheet_by_index(0)
    except Exception as e:
        print(f"⛔ 文件打开失败：{file_path}\n错误：{str(e)}")
        return 0

    params = []

    # SQL模板使用命名占位符
    sql_template = """
                   INSERT INTO t_yujin_crossborder_commodity_country
                   (`year_month`, hs_code, trade_type, country_code, country_name,
                    month_amount, cumulative_amount)
                   VALUES (:year_month, :hs_code, :trade_type, :country_code, :country_name, \
                           :month_amount, :cumulative_amount) ON DUPLICATE KEY \
                   UPDATE \
                       month_amount = \
                   VALUES (month_amount), cumulative_amount = \
                   VALUES (cumulative_amount) \
                   ,create_time = now()
                   """

    # 遍历数据行（从第7行开始）
    for row_idx in range(6, sheet.nrows):
        try:
            row = sheet.row_values(row_idx)
            if not row[1]:
                continue

            # 解析HS编码
            hs_code = parse_hscode(row[1])
            if not hs_code:
                continue  # 跳过无效行

            # 读取国家数据
            for col_idx in range(2, sheet.ncols, 2):
                country_name = sheet.cell_value(3, col_idx).strip()

                # 获取国家编码
                country_code = COUNTRY_CODE_MAPPING.get(country_name, 'XX')

                month_amount = parse_value(row[col_idx])
                cumulative_amount = parse_value(row[col_idx + 1]) if (col_idx + 1) < sheet.ncols else Decimal('0.0000')

                # 构建参数字典
                param_dict = {
                    "year_month": year_month,
                    "hs_code": hs_code,
                    "trade_type": trade_type,
                    "country_code": country_code,
                    "country_name": country_name,
                    "month_amount": month_amount,
                    "cumulative_amount": cumulative_amount
                }
                params.append(param_dict)

        except Exception as e:
            print(f"⛔ 行{row_idx}-列{col_idx}处理失败：{str(e)}")
            continue

    # 使用DBHelper执行批量SQL
    try:
        if params:
            row_count = db.execute_sql_with_params(sql_template, params)
            print(f"✅ 成功处理 {len(params)} 条记录，{trade_type}数据插入/更新 {row_count} 行")
            return row_count
        print(f"⚠️ 未找到有效数据：{file_path}")
        return 0
    except Exception as e:
        print(f"⛔ 数据库操作失败：{str(e)}")
        return 0


if __name__ == "__main__":
    # 测试用例
    export_file = r"D:\pythonSpace\crossborder\downloads\total\2025\04\(15)2025年对部分国家(地区)出口商品类章金额表.xls"
    import_file = r"D:\pythonSpace\crossborder\downloads\total\2025\04\(16)2025年自部分国家(地区)进口商品类章金额表.xls"

    # 处理出口数据
    parse_commodity_country_detail(export_file, "export")

    # 处理进口数据
    parse_commodity_country_detail(import_file, "import")