import re
from pathlib import Path

import xlrd


from crossborder.utils.db_helper import DBHelper
from crossborder.utils.parse_utils import parse_value, extract_year_month_from_path


def chinese_class_to_number(class_str):
    """中文类名转数字（保持原有实现）"""
    cn_num_map = {
        '一': 1, '二': 2, '三': 3, '四': 4, '五': 5, '六': 6,
        '七': 7, '八': 8, '九': 9, '十': 10, '十一': 11, '十二': 12,
        '十三': 13, '十四': 14, '十五': 15, '十六': 16, '十七': 17,
        '十八': 18, '十九': 19, '二十': 20, '二十一': 21, '二十二': 22
    }
    match = re.match(r'^第([一二三四五六七八九十百千万]+)类.*$', class_str)
    return cn_num_map.get(match.group(1), 0) if match else 0

def parse_commodity_table_excel(file_path):
    db = DBHelper()

    year, month = extract_year_month_from_path(Path(file_path))
    year_month = f"{year}-{month}"

    """处理单个月份文件，使用DBHelper执行SQL"""
    try:
        workbook = xlrd.open_workbook(file_path)
        sheet = workbook.sheet_by_index(0)
    except Exception as e:
        print(f"文件打开失败：{file_path}\n错误：{str(e)}")
        return 0

    params = []
    current_class = None

    # SQL模板使用命名占位符
    sql_template = """
        INSERT INTO `t_yujin_crossborder_commodity_trade`
        (`year_month`, `hs_code`, `monthly_export`, `ytd_export`,
         `monthly_import`, `ytd_import`, `ytd_yoy_export`, `ytd_yoy_import`)
        VALUES 
        (:year_month, :hs_code, :monthly_export, :ytd_export, 
         :monthly_import, :ytd_import, :ytd_yoy_export, :ytd_yoy_import)
        ON DUPLICATE KEY UPDATE
            monthly_export = VALUES(monthly_export),
            ytd_export = VALUES(ytd_export),
            monthly_import = VALUES(monthly_import),
            ytd_import = VALUES(ytd_import),
            ytd_yoy_export = VALUES(ytd_yoy_export),
            ytd_yoy_import = VALUES(ytd_yoy_import),
            create_time = NOW()   -- 这里改为直接使用NOW()
    """

    # 遍历数据行（从第7行开始）
    for row_idx in range(6, sheet.nrows):
        try:
            row = sheet.row_values(row_idx)
            cell_value = str(row[1]).strip() if len(row) > 1 else ""

            # 数据清洗
            clean_value = re.sub(r'\s+', ' ', cell_value)
            parts = clean_value.split(' ', 1)

            if not parts:
                continue

            identifier = parts[0]

            # 解析章数据
            if re.match(r'^\d+章$', identifier):
                if current_class:
                    chapter_num = re.findall(r'\d+', identifier)[0].zfill(2)
                    hs_code = f"{current_class}{chapter_num}"

                    # 构造参数字典
                    param_dict = {
                        "year_month": year_month,
                        "hs_code": hs_code,
                        "monthly_export": parse_value(row[2]),  # monthly_export
                        "ytd_export": parse_value(row[3]),      # ytd_export
                        "monthly_import": parse_value(row[4]),  # monthly_import
                        "ytd_import": parse_value(row[5]),      # ytd_import
                        "ytd_yoy_export": parse_value(row[6]),  # ytd_yoy_export
                        "ytd_yoy_import": parse_value(row[7])   # ytd_yoy_import
                    }
                    params.append(param_dict)

            # 解析类数据
            elif re.match(r'^第([一二三四五六七八九十百千万]+)类.*$', identifier):
                class_num = chinese_class_to_number(identifier)
                if 1 <= class_num <= 22:
                    current_class = f"{class_num:02d}"

                    # 类级别数据插入
                    param_dict = {
                        "year_month": year_month,
                        "hs_code": current_class,
                        "monthly_export": parse_value(row[2]),  # monthly_export
                        "ytd_export": parse_value(row[3]),      # ytd_export
                        "monthly_import": parse_value(row[4]),  # monthly_import
                        "ytd_import": parse_value(row[5]),      # ytd_import
                        "ytd_yoy_export": parse_value(row[6]),  # ytd_yoy_export
                        "ytd_yoy_import": parse_value(row[7])   # ytd_yoy_import
                    }
                    params.append(param_dict)

        except Exception as e:
            print(f"行{row_idx}处理失败：{str(e)}")
            continue

    # 使用DBHelper执行批量SQL
    try:
        if params:
            row_count = db.execute_sql_with_params(sql_template, params)
            print(f"成功处理 {len(params)} 条记录，插入/更新 {row_count} 行")
            return row_count
        return 0
    except Exception as e:
        print(f"数据库操作失败：{str(e)}")
        return 0

if __name__ == "__main__":
    file_path = r"D:\pythonSpace\crossborder\downloads\total\2025\04\(4)2025年进出口商品类章总值表.xls"
    result = parse_commodity_table_excel(file_path)
