import os
from decimal import Decimal
from pathlib import Path

import pandas as pd
import pymysql
from pymysql import Error


from crossborder.utils.constants import COUNTRY_CODE_MAPPING, EXCLUDE_REGIONS
from crossborder.utils.db_helper import DBHelper
from crossborder.utils.log import log
from crossborder.utils.parse_utils import parse_value, parse_ratio, extract_year_month_from_path


# 在代码中添加例外处理
def get_country_code(chinese_name):
    """带异常处理的国家编码查询"""
    code = COUNTRY_CODE_MAPPING.get(chinese_name.strip(), None)

    # 特殊处理逻辑
    if not code:
        if chinese_name.endswith("(地区)"):
            return "N/A"  # 标记为地区
        elif "国家联盟" in chinese_name:
            return "ORG"  # 标记为国际组织
    return code





def batch_upsert(file_path, year_month):
    """
    批量插入/更新国别贸易数据（使用 DBHelper）
    :param file_path: Excel 文件路径
    :param year_month: 年月标识（如 '2025-04'）
    """
    db = DBHelper()

    try:
        # 读取Excel文件
        df = pd.read_excel(file_path, engine='xlrd', header=None, dtype=str)
    except Exception as e:
        raise ValueError(f"文件读取失败：{str(e)}")

    # 定位数据起始行
    data_start = None
    for idx, row in df.iterrows():
        if "进出口商品国别" in str(row[1]):
            data_start = idx + 6
            break

    if not data_start:
        raise ValueError("无法定位数据起始行")

    # 读取数据区域
    df_data = pd.read_excel(file_path, engine='xlrd',
                            skiprows=data_start,
                            header=None,
                            usecols=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10])

    # 准备批量插入数据
    params_list = []
    for _, row in df_data.iterrows():
        country_name = str(row[1]).strip()

        # 过滤地区和组织
        if not country_name or country_name in EXCLUDE_REGIONS:
            continue

        # 获取国家代码
        country_code = get_country_code(country_name)
        if not country_code or country_code in ['N/A', 'ORG', 'XX']:
            continue

        # 数据转换（根据实际列索引调整）
        values = {
            'year_month': year_month,
            'country_code': country_code,
            'country_name': country_name,
            'monthly_total': parse_value(row[2]),  # monthly_total
            'monthly_export': parse_value(row[4]),  # monthly_export
            'monthly_import': parse_value(row[6]),  # monthly_import
            'ytd_total': parse_value(row[3]),      # ytd_total
            'ytd_export': parse_value(row[5]),     # ytd_export
            'ytd_import': parse_value(row[7]),     # ytd_import
            'yoy_import_export': parse_ratio(row[8]),  # yoy_import_export
            'yoy_export': parse_ratio(row[9]),      # yoy_export
            'yoy_import': parse_ratio(row[10])      # yoy_import
        }
        params_list.append(values)

    # 构建SQL模板（使用ON DUPLICATE KEY UPDATE）
    sql_template = """
        INSERT INTO t_yujin_crossborder_country_trade 
        (`year_month`, country_code, country_name, monthly_total, monthly_export, monthly_import, 
         ytd_total, ytd_export, ytd_import, yoy_import_export, yoy_export, yoy_import, create_time)
        VALUES 
        (:year_month, :country_code, :country_name, :monthly_total, :monthly_export, :monthly_import, 
         :ytd_total, :ytd_export, :ytd_import, :yoy_import_export, :yoy_export, :yoy_import, NOW())
        ON DUPLICATE KEY UPDATE
            monthly_total = VALUES(monthly_total),
            monthly_export = VALUES(monthly_export),
            monthly_import = VALUES(monthly_import),
            ytd_total = VALUES(ytd_total),
            ytd_export = VALUES(ytd_export),
            ytd_import = VALUES(ytd_import),
            yoy_import_export = VALUES(yoy_import_export),
            yoy_export = VALUES(yoy_export),
            yoy_import = VALUES(yoy_import),
            create_time = VALUES(create_time)
    """

    # 调用 DBHelper 的 execute_sql_with_params 方法
    affected_rows = db.execute_sql_with_params(sql_template, params_list)
    log.info(f"✅ 成功更新 {year_month}，影响 {affected_rows} 条记录")
    return affected_rows


def parse_country_table_excel(file_path):
    """
    解析并入库国别贸易表（单文件）
    :param file_path: Excel 文件路径
    """
    log.info(f"🏁 商品国别贸易清洗：{file_path}")

    # 从路径中提取年份和月份

    year, month = extract_year_month_from_path(Path(file_path))
    year_month = f"{year}-{month}"

    # 执行批量插入
    try:
        count = batch_upsert(file_path, year_month)
        log.info(f"✅ 文件 {file_path} 处理完成，共影响 {count} 条记录")
        return count
    except Exception as e:
        log.error(f"❌ 文件 {file_path} 处理失败：{str(e)}")
        raise


if __name__ == "__main__":
    file_path = r"D:\pythonSpace\crossborder\downloads\total\2025\03\(2)2025年进出口商品国别(地区)总值表.xls"
    parse_country_table_excel(file_path)