from datetime import datetime

from crossborder.utils.db_helper import DBHelper
import xlrd
import re

from crossborder.utils.log import log
from crossborder.utils.parse_utils import convert_unit, parse_ratio

CURRENT_YEAR = str(datetime.now().year)

def is_current_year_data(date_str):
    """
    判断是否为当前年份的数据（如 2025.01）
    :param date_str: 年月字符串，如 "2025.01"
    :return: bool
    """
    try:
        return date_str.startswith(f"{CURRENT_YEAR}.")
    except:
        return False




def generate_sql(data_group):
    """动态生成SQL语句（仅保留核心字段提取）"""
    base_row = data_group[0]
    year_month = base_row[1].replace('.', '-')

    # 提取基础字段
    monthly_total = convert_unit(base_row[2])
    monthly_import = convert_unit(base_row[4])
    monthly_export = convert_unit(base_row[3])
    trade_balance = convert_unit(base_row[5])
    ytd_total = convert_unit(base_row[6])
    ytd_import = convert_unit(base_row[8])
    ytd_export = convert_unit(base_row[7])
    ytd_trade_balance = convert_unit(base_row[9])

    # 初始化同比数据
    yoy_import_export = yoy_import = yoy_export = None

    # 处理2025年三行结构
    if len(data_group) == 3:
        yoy_row = data_group[1]
        yoy_import_export = parse_ratio(yoy_row[2])
        yoy_import = parse_ratio(yoy_row[4])
        yoy_export = parse_ratio(yoy_row[3])

    return {
        'year_month': year_month,
        'monthly_total': monthly_total,
        'monthly_import': monthly_import,
        'monthly_export': monthly_export,
        'trade_balance': trade_balance,
        'ytd_total': ytd_total,
        'ytd_import': ytd_import,
        'ytd_export': ytd_export,
        'ytd_trade_balance': ytd_trade_balance,
        'yoy_import_export': yoy_import_export,
        'yoy_import': yoy_import,
        'yoy_export': yoy_export
    }


def parse_month_table_excel(file_path):
    """
    解析月度汇总表并入库
    :param file_path: Excel 文件路径
    """
    log.info(f"月度表数据解析：{file_path}")
    db = DBHelper()
    sql_template = """
    INSERT INTO t_yujin_crossborder_monthly_summary 
    (`year_month`, monthly_total, monthly_import, monthly_export, trade_balance,
     ytd_total, ytd_import, ytd_export, ytd_trade_balance,
     yoy_import_export, yoy_import, yoy_export, create_time)
    VALUES 
    (:year_month, :monthly_total, :monthly_import, :monthly_export, :trade_balance,
     :ytd_total, :ytd_import, :ytd_export, :ytd_trade_balance,
     :yoy_import_export, :yoy_import, :yoy_export, NOW())
    ON DUPLICATE KEY UPDATE
        monthly_total = VALUES(monthly_total),
        monthly_import = VALUES(monthly_import),
        monthly_export = VALUES(monthly_export),
        trade_balance = VALUES(trade_balance),
        ytd_total = VALUES(ytd_total),
        ytd_import = VALUES(ytd_import),
        ytd_export = VALUES(ytd_export),
        ytd_trade_balance = VALUES(ytd_trade_balance),
        yoy_import_export = VALUES(yoy_import_export),
        yoy_import = VALUES(yoy_import),
        yoy_export = VALUES(yoy_export),
        create_time = VALUES(create_time)
    """

    try:
        workbook = xlrd.open_workbook(file_path)
        sheet = workbook.sheet_by_index(0)

        row_idx = 5  # 数据起始行
        params_list = []

        while row_idx < sheet.nrows:
            # 获取基础行
            base_row = sheet.row_values(row_idx)
            date_cell = str(base_row[1])

            # 校验是否是有效日期格式：如 "2025.01"
            if not re.match(r"202[0-9]\.\d{2}", date_cell):
                row_idx += 1
                continue

            # 动态读取数据组
            if is_current_year_data(date_cell):
                try:
                    data_group = [
                        base_row,
                        sheet.row_values(row_idx + 1),
                        sheet.row_values(row_idx + 2)
                    ]
                    step = 3
                except IndexError:
                    print(f"行 {row_idx} 数据不完整，跳过")
                    row_idx += 1
                    continue
            else:
                data_group = [base_row]
                step = 1

            # 构造参数
            data = generate_sql(data_group)
            params_list.append(data)
            row_idx += step

        # 批量执行 SQL 插入
        if params_list:
            db.execute_sql_with_params(sql_template, params_list)
            log.info(f"成功处理 {len(params_list)} 条数据，已入库。")
        else:
            log.error("未找到可处理的数据")

    except Exception as e:
        log.error(f"Excel解析失败: {e}")
        raise


if __name__ == "__main__":
    file_path = r"D:\pythonSpace\crossborder\downloads\total\2025\03\(1)2025年进出口商品总值表 B-月度表.xls"
    parse_month_table_excel(file_path)