from pathlib import Path

import xlrd

from crossborder.utils.constants import REGION_MAPPING
from crossborder.utils.db_helper import DBHelper
from crossborder.utils.parse_utils import extract_year_month_from_path, parse_value


def parse_region_table_excel(file_path):
    """处理地区数据表，使用DBHelper执行SQL"""
    db = DBHelper()

    # 从文件路径提取年份和月份
    year, month = extract_year_month_from_path(Path(file_path))
    year_month = f"{year}-{month:02d}"

    try:
        workbook = xlrd.open_workbook(file_path)
        sheet = workbook.sheet_by_index(0)
    except Exception as e:
        print(f"文件读取失败：{file_path}\n错误：{str(e)}")
        return 0

    params = []

    # SQL模板使用命名占位符
    sql_template = """
                   INSERT INTO `t_yujin_crossborder_region_trade`
                   (`year_month`, `region_code`, `region_name`, `region_type`,
                    `monthly_total`, `monthly_export`, `monthly_import`,
                    `ytd_total`, `ytd_export`, `ytd_import`)
                   VALUES (:year_month, :region_code, :region_name, :region_type, \
                           :monthly_total, :monthly_export, :monthly_import, \
                           :ytd_total, :ytd_export, :ytd_import) ON DUPLICATE KEY \
                   UPDATE \
                       region_name = \
                   VALUES (region_name), region_type = \
                   VALUES (region_type), monthly_total = \
                   VALUES (monthly_total), monthly_import = \
                   VALUES (monthly_import), monthly_export = \
                   VALUES (monthly_export), ytd_total = \
                   VALUES (ytd_total), ytd_import = \
                   VALUES (ytd_import), ytd_export = \
                   VALUES (ytd_export) \
                   ,create_time = NOW()
                   """

    # 从第7行开始读取（索引6）
    for row_idx in range(6, sheet.nrows):
        try:
            row = sheet.row_values(row_idx)
            region_name = str(row[1]).strip()

            # 跳过空行和标题行
            if not region_name or region_name == "地区":
                continue

            # 精确匹配行政区划
            region_info = REGION_MAPPING.get(region_name, None)
            if not region_info:
                # 尝试模糊匹配（处理名称变体）
                region_info = next(
                    (v for k, v in REGION_MAPPING.items() if region_name in k),
                    None
                )
                if not region_info:
                    print(f"⚠️ 地区匹配失败：{region_name}")
                    continue

            # 构建参数字典
            param_dict = {
                "year_month": year_month,
                "region_code": region_info['code'],  # region_code
                "region_name": region_name,
                "region_type": region_info['type'],
                "monthly_total": parse_value(row[2]),  # monthly_total
                "monthly_import": parse_value(row[4]),  # monthly_import
                "monthly_export": parse_value(row[6]),  # monthly_export
                "ytd_total": parse_value(row[8]),  # ytd_total
                "ytd_import": parse_value(row[9]),  # ytd_import
                "ytd_export": parse_value(row[10])  # ytd_export
            }
            params.append(param_dict)

        except Exception as e:
            print(f"行{row_idx}处理失败：{str(e)}")
            continue

    # 使用DBHelper执行批量SQL
    try:
        if params:
            row_count = db.execute_sql_with_params(sql_template, params)
            print(f"✅ 成功处理 {len(params)} 条记录，插入/更新 {row_count} 行")
            return row_count
        print(f"⚠️ 未找到有效数据：{file_path}")
        return 0
    except Exception as e:
        print(f"数据库操作失败：{str(e)}")
        return 0


if __name__ == "__main__":
    # 测试用文件路径
    file_path = r"D:\pythonSpace\crossborder\downloads\total\2025\04\(8)2025年进出口商品收发货人所在地总值表.xls"
    result = parse_region_table_excel(file_path)
    print(f"处理结果：{result}")