import argparse
import os
import random
import re
import time
from datetime import datetime
from pathlib import Path

from faker import Faker
from selenium import webdriver
from selenium.common import StaleElementReferenceException
from selenium.webdriver import FirefoxOptions, ActionChains
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

from crossborder.quanguo.parse_country_table_excel import parse_country_table_excel
from crossborder.quanguo.parse_month_excel import parse_month_table_excel
from crossborder.quanguo.parse_year_excel import parse_year_table_excel
from crossborder.utils.constants import DOWNLOAD_DIR
from crossborder.utils.download_utils import configure_stealth_options, wait_for_download, download_excel
from selenium.webdriver.common.by import By

from crossborder.utils.log import log

base_url = "http://www.customs.gov.cn/customs/302249/zfxxgk/2799825/302274/302277/6348926/index.html"
download_dir = DOWNLOAD_DIR / "total"




def generate_table_title(year):
    return [
        f"(1){year}年进出口商品总值表 A:年度表",
        f"(1){year}年进出口商品总值表 B:月度表",
        f"(2){year}年进出口商品国别(地区)总值表",
        f"(4){year}年进出口商品类章总值表",
        f"(8){year}年进出口商品收发货人所在地总值表",
        f"(15){year}年对部分国家(地区)出口商品类章金额表",
        f"(16){year}年自部分国家(地区)进口商品类章金额表"
    ]

def process_table_row(row):
    """动态处理表格行数据（Selenium语法）"""
    try:
        # 获取所有表格单元格(td)元素
        cells = row.find_elements(By.TAG_NAME, 'td')
        if len(cells) < 2:
            return None

        # 获取表格名
        table_name = cells[0].text.strip()

        # 获取第二列中的所有链接，提取月份和href
        month_links = []
        links = cells[1].find_elements(By.TAG_NAME, 'a')
        for a in links:
            # 获取文本并去掉‘月’
            month_text = a.text
            if '月' in month_text:
                month = int(month_text.replace('月', '').strip())
                href = a.get_attribute('href')
                if href:
                    month_links.append((month, href))

        # 按月份倒序排列（1-12月）
        month_links.sort(key=lambda x: x[0], reverse=True)

        return (table_name, month_links)

    except Exception as e:
        log.info(f"表格行处理异常: {str(e)}")
        return None


def go_to_year_page(driver, year):
    """切换到指定年份页面"""
    try:
        # 使用 XPath 定位 id="one1" 并且文本等于 year 的 <a>
        xpath = f'//div[@class="tjYear"]/a[normalize-space()="{year}"]'
        year_link = WebDriverWait(driver, 20).until(
            EC.element_to_be_clickable((By.XPATH, xpath))
        )
        driver.execute_script("arguments[0].scrollIntoView({block: 'center'});", year_link)
        # 使用 ActionChains 安全点击
        ActionChains(driver).move_to_element(year_link).pause(0.3).click().perform()
        WebDriverWait(driver, 30).until(
            lambda d: d.execute_script("return document.readyState") == "complete"
        )
        log.info(f"已切换到 {year} 页面")
        return True
    except Exception as e:
        log.error(f"无法切换到 {year} 页面: {str(e)}")
        return False


def crawl_with_selenium(driver, base_url, year, latest_only=False):
    """主抓取函数"""
    driver.get(base_url)

    if not go_to_year_page(driver, year):
        log.warning(f"{year} 页面不可用，跳过")
        return
    log.info(f"开始抓取 {year} 年数据:{driver.current_url}")
    try:
        while True:
            table = WebDriverWait(driver, 20).until(
                EC.presence_of_element_located((By.CSS_SELECTOR, f"#yb{year}RMB"))
            )
            current_rows = table.find_elements(By.CSS_SELECTOR, "tr:not(:first-child)")

            if not current_rows:
                log.info(f"{year}年所有表格处理完成")
                break

            row = current_rows[0]
            result = process_table_row(row)
            table_name, month_links = result or (None, [])

            table_title = generate_table_title(year)

            if result and result[0] in table_title:
                handle_month_data(driver, sanitize_filename(table_name), month_links, year=year, latest_only=latest_only)

            driver.execute_script("arguments[0].remove()", row)
            WebDriverWait(driver, 10).until(EC.staleness_of(row))
            time.sleep(random.uniform(1, 3))

    except StaleElementReferenceException:
        log.info("检测到元素失效，自动刷新表格")
        driver.refresh()
        WebDriverWait(driver, 30).until(
            EC.presence_of_element_located((By.CSS_SELECTOR, f"#yb{year}RMB"))
        )

def sanitize_filename(filename):
    # 替换所有非法字符为空或合法字符（如用 - 替代）
    return re.sub(r'[<>:"/\\|?*\x00-\x1F]', '-', filename)


def handle_month_data(driver, table_name, month_links, year, latest_only):
    main_window = driver.current_window_handle
    for idx, month_data in enumerate(month_links):
        if 1 <= month_data[0] <= 12:
            # 新标签页策略（防止主页面DOM变更）
            driver.switch_to.window(main_window)
            driver.execute_script(f"window.open('{month_data[1]}', '_blank_{idx}')")
            driver.switch_to.window(driver.window_handles[-1])

            month_num, link = month_data
            try:
               download_excel(driver, link, year, month_num, table_name, download_dir)
            except Exception as e:
               log.info(f"【异常】下载失败: {str(e)}")
            time.sleep(random.uniform(0.5, 1.5))  # 下载间隔



if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="抓取海关总署年度数据")
    parser.add_argument("--year", type=int, help="起始年份，例如：--year 2023 表示抓取 2023 和 2024")
    args = parser.parse_args()

    current_year = datetime.now().year
    start_year = args.year if args.year else current_year
    years_to_crawl = list(range(start_year, current_year + 1))
    years_to_crawl.reverse()


    log.info(f"即将抓取 {start_year} - {current_year} 年度数据")
    options = configure_stealth_options(download_dir)
    driver = webdriver.Firefox(options=options)

    try:
        for year in years_to_crawl:
            base_url = "http://www.customs.gov.cn/customs/302249/zfxxgk/2799825/302274/302277/6348926/index.html"
            log.info(f"\n【{year}年】开始抓取...".center(66, "-"))
            crawl_with_selenium(driver, base_url, year=year, latest_only=args.year is None)
    finally:
        driver.quit()
        log.info("【海关总署】全年数据抓取结束".center(66, "*"))
        log.info("\n数据清洗入库中...")
        # perform_data_cleanup_and_import(current_year)